# Copyright 2004-2005 swissnex
# 
# This file is part of EventRegistration.
# 
# EventRegistration is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# EventRegistration is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with EventRegistration; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
# 
# 
# $Id: Event.py 89 2005-04-18 21:02:19Z jason $

'''
custom event type allowing users to register to attend events
'''


from Products.Archetypes.public import BaseFolderSchema
from Products.Archetypes.public import Schema
from Products.Archetypes.public import StringField
from Products.Archetypes.public import TextField
from Products.Archetypes.public import DateTimeField
from Products.Archetypes.public import ComputedField
from Products.Archetypes.public import StringWidget
from Products.Archetypes.public import MultiSelectionWidget
from Products.Archetypes.public import SelectionWidget
from Products.Archetypes.public import TextAreaWidget
from Products.Archetypes.public import CalendarWidget
from Products.Archetypes.public import RichWidget
from Products.Archetypes.public import EpozWidget
from Products.Archetypes.public import BaseFolder
from Products.Archetypes.public import registerType
from Products.Archetypes.public import *
from Products.Archetypes.TemplateMixin import TemplateMixin
from Products.CMFCore import CMFCorePermissions
from DateTime import DateTime

from Products.EventRegistration.config import PROJECTNAME
from Products.EventRegistration.utils import getDisplayList

from AccessControl import ClassSecurityInfo


# debugging needs
from Products.Archetypes.debug import log
from pprint import pformat

security = ClassSecurityInfo()

schema = BaseFolderSchema.copy() + Schema((
	StringField(
		'id',
		required = True,
		widget = StringWidget(
			label = 'Short Name',
			description = 'Short Name is part of the item\'s web address. (For example, a short name of "big-art-show" produces a web address of "http://web.swissnex.org/events/big-art-show".)  Do not use spaces, upper case, underscores, or special characters.',
			),
		),
	StringField(
		'title',
		required = True,
		widget = StringWidget(
			label = 'Title',
			description = ' ',
			),
		),
	StringField(
		'event_location',
		required = True,
		vocabulary='EventLocations',
		widget = SelectionWidget(
			label='Location',
			description = 'Where the event will be held',
			format = 'select',
			),
		),
	TextField('summary',
		searchable = True,
		required = True,
		widget = TextAreaWidget(
			label = 'Summary',
			description = 'Brief description of the event, displayed on the home page events listing',
			),
		),
	DateTimeField('start_date',
		default=DateTime(),
		required = True,
		searchable = True,
		widget=CalendarWidget(
			label="Event Starts",
			description = 'Date event begins',
			),
		),
	DateTimeField('end_date',
		default=DateTime(),
		required = True,
		searchable = True,
		widget=CalendarWidget(
			label="Event Ends",
			description = 'Date event ends',
			),
		),
	DateTimeField('effective_date',
		required = False,
		searchable = False,
		widget = CalendarWidget(
			label = 'Effective Date',
			description = 'Date event will appear on website (will appear immediately upon publication if left blank)',
			),
		),
	DateTimeField('expires_date',
		required = False,
		searchable = False,
		widget = CalendarWidget(
			label = 'Expiration Date',
			description = 'Date event will no longer appear in past events listing (will be available forever if left blank)',
			),
		),
	TextField(
		'full_description',
		searchable = True,
		required = True,
		default_output_type = 'text/html',
		allowable_content_types = (
			'text/html',
			),
		widget = RichWidget(
			label = 'Full Description',
			description = ' ',
			),
		),
))


class Event(BaseFolder):
	'''An event that members of the public may register to attend.'''

	filter_content_types = True
	allowed_content_types = ['Registrant',]
	content_icon = 'event_icon.gif'
	schema = schema

	actions = (
			{
			'id'           : 'contents',
			'name'         : 'Show Registrants',
			'permissions'  : (CMFCorePermissions.ModifyPortalContent, ),
			'action'       : 'string:${object_url}/folder_contents',
			},
			{
			'id'           : 'export',
			'name'         : 'Export List of Registrants',
			'permissions'  : (CMFCorePermissions.ModifyPortalContent, ),
			'action'       : 'string:${object_url}/export_registrants',
			},
			)


	def EventLocations(self):
		'''get the vocabulary from event_registration_properties'''
		return getDisplayList(self, 'event_locations')

	def start(self):
		""" the 'start_date' as DateTime object or 'created()' if not set"""
		sd = getattr(self, 'start_date', None)
		return sd or self.created()

	def end(self):
		""" the 'end_date' as DateTime object or 'start()' if not set"""
		ed = getattr(self, 'end_date', None)
		return ed or self.start()

	# put some metadata on the edit form:
	security.declarePublic('getSummary')
	def getSummary(self):
		return str(self.Description())
	def setSummary(self, val):
		self.summary = val
		if val:
			return str(self.setDescription(val))
		else:
			return self.Description()
	def getEffective_date(self):
		return self.EffectiveDate()
	def setEffective_date(self, val):
		self.effective_date = val
		if val:
			return self.setEffectiveDate(val)
		else:
			return self.EffectiveDate()
	def getExpires_date(self):
		return self.ExpirationDate()
	def setExpires_date(self, val):
		self.expires_date = val
		if val:
			return self.setExpirationDate(val)
		else:
			return self.ExpirationDate()

def modify_fti(fti):
	for a in fti['actions']:
		if a['id'] in ('references', ):
			a['visible'] = False
		if a['id'] in ('view', ):
			a['action'] = 'string:${object_url}/event_view'
	return fti

registerType(Event)
